/*
 * Copyright (C) 2014 digitalSTROM.org, Schlieren, Switzerland
 *
 * digitalSTROM ds485-base library is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the License,
 * or (at your option) any later version.
 *
 * digitalSTROM ds485-base library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with digitalSTROM ds485-base library. If not, see
 * <http://www.gnu.org/licenses/>.
 */

#include "ds_string.h"

#include <errno.h>
#include <stdio.h>

static char dsid_format[] = "%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx";
int scanf_dsid(const char *buf, dsid_t *dsid)
{
    int ret;
    ret = sscanf(buf, dsid_format,
           &dsid->id[0],
           &dsid->id[1],
           &dsid->id[2],
           &dsid->id[3],
           &dsid->id[4],
           &dsid->id[5],
           &dsid->id[6],
           &dsid->id[7],
           &dsid->id[8],
           &dsid->id[9],
           &dsid->id[10],
           &dsid->id[11]);

    if (ret != sizeof(dsid->id)) {
        return -EINVAL;
    }
    return 0;
}

int sprintf_container(char *buf, const ds485_container_t *container)
{
    int n;
    char *p;

    p = buf;
    p += sprintf(p, "C type:0x%x tid:0x%02X len:%02d ",
                 container->containerType, container->transactionId,
                 container->length);
    p += dsuid_to_string(&container->sourceId, p);
    p += sprintf(p, "->");
    p += dsuid_to_string(&container->destinationId, p);
    p += sprintf(p, " d:");
    for (n = 0; n < 16 && n < container->length; n++)
        p += sprintf(p, "%02x", container->data[n]);
    *p = 0;

    return (p - buf);
}

int sprintf_netlib_packet(char *buf, const ds485n_packet_t *packet)
{
    int n;
    char *p;

    p = buf;
    p += sprintf(p, "L type:0x%x tid:0x%02X len:%02d ", packet->containerType,
                 packet->transactionId, packet->length);
    p += dsuid_to_string(&packet->sourceId, p);
    p += sprintf(p, "->");
    p += dsuid_to_string(&packet->destinationId, p);
    p += sprintf(p, " d:");
    for (n = 0; n < 16 && n < packet->length; n++)
        p += sprintf(p, "%02x", packet->data[n]);
    *p = 0;

    return (p - buf);
}

#if 0
int main()
{
    DSID_STR_BUF(buf);
    dsid_t dsid;
    scanf_dsid("123456789012345678901234", &dsid);
    sprintf_dsid(buf, &dsid);
    printf("dsid: %s\n", buf);

}
#endif
